/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.data.api

import net.moonleay.lilJudd.data.api.entry.schedule.ModeData
import net.moonleay.lilJudd.util.TimeUtil

object Splatoon3Api {
    private fun getRegularMode(timestamp: Long): ModeData {
        Splatoon3ApiCache.cachedRegularModeData.map { modeData ->
            val startTime = TimeUtil.deformatJSONTime(modeData.startTime, "UTC")
            val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
            if (timestamp in startTime..endTime) {
                return modeData
            }
        }
        throw Exception("No current mode found")
    }

    private fun getOpenMode(timestamp: Long): ModeData {
        Splatoon3ApiCache.cachedCompetitiveOpenModeData.map { modeData ->
            val startTime = TimeUtil.deformatJSONTime(modeData.startTime, "UTC")
            val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
            if (timestamp in startTime..endTime) {
                return modeData
            }
        }
        throw Exception("No current mode found")
    }

    private fun getXMode(timestamp: Long): ModeData {
        Splatoon3ApiCache.cachedXModeData.map { modeData ->
            val startTime = TimeUtil.deformatJSONTime(modeData.startTime, "UTC")
            val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
            if (timestamp in startTime..endTime) {
                return modeData
            }
        }
        throw Exception("No current mode found")
    }

    private fun getSeriesMode(timestamp: Long): ModeData {
        Splatoon3ApiCache.cachedCompetitiveSeriesModeData.map { modeData ->
            val startTime = TimeUtil.deformatJSONTime(modeData.startTime, "UTC")
            val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
            if (timestamp in startTime..endTime) {
                return modeData
            }
        }
        throw Exception("No current mode found")
    }

    fun getRegularMapsFormatted(timestamp: Long): String {
        val modeData = getRegularMode(timestamp)
        val map1 = modeData.map1!!.name.split(" ")[0]
        val map2 = modeData.map2!!.name.split(" ")[0]
        return "R: $map1 & $map2"
    }

    fun getOpenModeFormatted(timestamp: Long): String {
        val modeData = getOpenMode(timestamp)
        val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
        val diffStamp = TimeUtil.getTimeDifferenceFormatted(System.currentTimeMillis(), endTime)
        return "O: ${modeData.ruleSetName} $diffStamp left"
    }

    fun getOpenMapFormatted(timestamp: Long): String {
        val modeData = getOpenMode(timestamp)
        val map1 = modeData.map1!!.name.split(" ")[0]
        val map2 = modeData.map2!!.name.split(" ")[0]
        return "O: $map1 & $map2"
    }

    fun getSeriesModeFormatted(timestamp: Long): String {
        val modeData = getSeriesMode(timestamp)
        val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
        val diffStamp = TimeUtil.getTimeDifferenceFormatted(System.currentTimeMillis(), endTime)
        return "S: ${modeData.ruleSetName} $diffStamp left"
    }

    fun getSeriesMapsFormatted(timestamp: Long): String {
        val modeData = getSeriesMode(timestamp)
        val map1 = modeData.map1!!.name.split(" ")[0]
        val map2 = modeData.map2!!.name.split(" ")[0]
        return "S: $map1 & $map2"
    }

    fun getXModeFormatted(timestamp: Long): String {
        val modeData = getXMode(timestamp)
        val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
        val diffStamp = TimeUtil.getTimeDifferenceFormatted(System.currentTimeMillis(), endTime)
        return "X: ${modeData.ruleSetName} $diffStamp left"
    }

    fun getXMapFormatted(timestamp: Long): String {
        val modeData = getXMode(timestamp)
        val map1 = modeData.map1!!.name.split(" ")[0]
        val map2 = modeData.map2!!.name.split(" ")[0]
        return "X: $map1 & $map2"
    }
}
