/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.application.slash.converters.impl.enumChoice
import com.kotlindiscord.kord.extensions.commands.converters.impl.channel
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import com.kotlindiscord.kord.extensions.utils.hasPermission
import dev.kord.common.entity.Permission
import net.moonleay.lilJudd.extensions.component.EnableOrDisable
import net.moonleay.lilJudd.features.component.FeatureEnum
import net.moonleay.lilJudd.features.component.FeatureManager
import net.moonleay.lilJudd.util.EmbedColor
import net.moonleay.lilJudd.util.Logger
import net.moonleay.lilJudd.util.MessageUtil

class FeatureManageExtension : Extension() {

    override val name = "feature"
    override val allowApplicationCommandInDMs: Boolean
        get() = false


    override suspend fun setup() {
        publicSlashCommand(::FeatureManagerArgs) {
            name = "feature"
            description = "Manage features"

            this.action {
                val u = this.user
                if (!u.asMember(this.guild!!.id).hasPermission(Permission.Administrator)) {
                    this.respond {
                        embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "403: Forbidden",
                                "You cannot edit features, as you don't have the Administrator permission.",
                                u.asUser().username + "#" + u.asUser().discriminator
                            )
                        )
                    }
                    return@action
                }
                val gID = this.guild!!.id.value.toLong()
                val cID = this.arguments.channel.id.value.toLong()
                val channel = this.arguments.channel
                val args = this.arguments
                Logger.out("${args.feature.readableName} ${args.setStatus.readableName} ${channel.data.name.value}")
                val f = FeatureManager.getFeature(args.feature)
                if (f == null) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "404: Not Found",
                                "The feature you are trying to edit does not exist.",
                                u.asUser().username + "#" + u.asUser().discriminator
                            )
                        )
                    }
                    return@action
                }

                if (this.arguments.setStatus == EnableOrDisable.ENABLE) {
                    this.respond {
                        this.embeds.add(f.enable(u, gID, cID, channel, args))
                    }
                    return@action
                }
                this.respond {
                    this.embeds.add(f.disable(u, gID, cID, channel, args))
                }
            }
        }
    }

    inner class FeatureManagerArgs : Arguments() {

        val feature by enumChoice<FeatureEnum> {
            this.name = "feature"
            this.description = "The targeted feature"
            this.typeName = "en_US"
        }

        val setStatus by enumChoice<EnableOrDisable> {
            this.name = "set"
            this.description = "Set enabled or disabled"
            this.typeName = "en_US"
        }

        val channel by channel {
            this.name = "channel"
            this.description = "Target Channel"
        }
    }
}
