/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.jobs

import dev.inmo.krontab.KronScheduler
import kotlinx.coroutines.Job
import net.moonleay.lilJudd.data.api.Splatoon3ApiCache
import net.moonleay.lilJudd.data.api.type.ApiDataType
import net.moonleay.lilJudd.data.api.type.ApiRequestType
import net.moonleay.lilJudd.jobs.component.CronjobType
import net.moonleay.lilJudd.jobs.component.ICronjob
import net.moonleay.lilJudd.util.Logger

object Splatoon3ApiFestivalAndCoopUpdateScheduler : ICronjob {
    override val jobName: String
        get() = "Splatoon3ApiFestivalAndCoopUpdateScheduler"
    override val jobIncoming: String
        get() = "0 0 0 /1 * * 0o *" // once a day
    override val jobType: CronjobType
        get() = CronjobType.INFINITE
    override val continueJob: Boolean
        get() = true
    override lateinit var cronjobJob: Job
    override lateinit var scheduler: KronScheduler

    override suspend fun jobFunction() {
        Logger.out("Running Splatoon3ApiFestivalUpdateScheduler.")
        Splatoon3ApiCache.updateData(ApiDataType.SPLATFESTS, ApiRequestType.AUTOMATIC_CACHE_UPDATE)
        Logger.out("Splatoon3ApiFestivalUpdateScheduler finished.")
        Logger.out("Running Splatoon3ApiCoopUpdateScheduler.")
        Splatoon3ApiCache.updateData(ApiDataType.COOP, ApiRequestType.AUTOMATIC_CACHE_UPDATE)
        Logger.out("Splatoon3ApiCoopUpdateScheduler finished.")
    }
}
